@@ -5,7 +5,8 @@ import shutil
 import sys
 import random
 import codecs
-import pickle
+import dill
+import json
 
 from importlib import util
 from dotenv import load_dotenv
@@ -16,40 +17,20 @@ sandy_base_url = "https://app.daisi.io"
 headers = {"Authorization": "token " + os.getenv("ACCESS_TOKEN", "")}
 
 
-def load_pickle_string(s):  # pragma: no cover
-    return pickle.loads(codecs.decode(s.encode(), "base64"))
-
-#
-def parse_hidden(output):
-    final_output=[]
-    for out in [y["data"] for y in output["outputs"]]:
-    
-        if type(out) == str and out.startswith("lookup:"):
-            # Get the binary data
-            l_split = out.split("lookup:")
-
-            r = requests.get(
-                sandy_base_url + "/pickle?lookup=" + l_split[1], headers=headers
-            )
-
-            out = load_pickle_string(r.content.decode())
-            final_output.append(out)
-        else:
-            final_output.append(out)
-    # for y in output:
-    #     try:
-    #         id = y['id']
-    #         r = requests.get(
-    #             sandy_base_url + "/pickle?lookup=" + id, headers=headers
-    #         )
-
-    #         out = load_pickle_string(r.content.decode())
-    #         final_output.append(out)
-    #     except:
-    #         final_output.append(y)
+def load_dill_string(s):  # pragma: no cover
+    return dill.loads(codecs.decode(s.encode(), "base64"))
 
 
-    return final_output
+def get_dill_string(obj):  # pragma: no cover
+    return codecs.encode(dill.dumps(obj), "base64").decode()
+
+
+def is_jsonable(x):
+    try:
+        json.dumps(x)
+        return True
+    except (TypeError, OverflowError):
+        return False
 
 
 class Daisi:
@@ -74,6 +55,7 @@ class Daisi:
         self.name = None
         self.description = None
         self.base_url = base_url + "/pebble-cli-api/pebbles"
+        self.data_mapping = {}
 
         if not daisi_id:
             self.name = self.random_name()
@@ -111,48 +93,18 @@ class Daisi:
                 raise ValueError("That daisi could not be found.")
 
 
-    def chain(self, args):
-        """
-        Collects reusable outputs from the output of another daisi.
-
-        :param args: A dictionary of arguments used in the chain.
-        :type args: dict
-
-        :return: A clean dictionary to be used as daisi input
-        :rtype dict
-        """
-        # Set all the arguments as provided
-        clean_args = args
-
-        # Parse the arguments
-        # for argument_name, argument_value in args.items():
-
-        #     # If the argument is a daisi output, we must chain the data
-        #     if type(argument_value) == list:
-
-        #         # Assume it's the first output
-        #         try:
-        #             clean_args[argument_name] = argument_value[0]["data"]
-
-        #             # Check whether the ids of outputs match
-        #             for arg in argument_value:
-
-        #                 # If the id of a particular output matches the argument name, set the value
-        #                 if arg["id"].startswith(argument_name) or argument_name.startswith(
-        #                     arg["id"]
-        #                 ):
-        #                     clean_args[argument_name] = arg["data"]
-        #         except:
-        #             continue
-
-        return clean_args
-
-
-    def compute(self, func="compute", **args):
+    def compute(self, file=None, func="compute", **args):
         # Check whether any of the arguments are daisis
         parsed_args = self.chain(args)
+        parsed_args = self.pickle_hidden(args)
+
+        # Include the function as the endpoint, if provided
         parsed_args["_endpoint"] = func
 
+        # Only include the file argument if it's not None
+        if file:
+            parsed_args["_file"] = file
+
         # Call the specified Daisi compute
         r = requests.post(
             self.base_url + "/" + self.uuid + "/compute", json=parsed_args, headers=headers
@@ -160,7 +112,7 @@ class Daisi:
 
         result = None
         if r.status_code == 200:
-            result = parse_hidden(r.json())
+            result = self.unpickle_hidden(r.json())
 
         return result
 
@@ -182,6 +134,50 @@ class Daisi:
         return result
 
 
+    def unpickle_hidden(self, output):
+        final_output=[]
+        for out in [y["data"] for y in output["outputs"]]:
+            if type(out) == str and out.startswith("lookup:"):
+                # Get the binary data
+                l_split = out.split("lookup:")
+
+                r = requests.get(
+                    self.base_url + "/pickle?lookup=" + l_split[1], headers=headers
+                )
+
+                out = load_dill_string(r.content.decode())
+
+                final_output.append(out)
+            else:
+                final_output.append(out)
+
+        return final_output
+
+
+    def store_pickle(self, data):
+        my_args = {"data": data}
+
+        # Call the specified Daisi compute
+        r = requests.post(
+            self.base_url + "/pickle", json=my_args, headers=headers
+        )
+
+        # Return the result
+        return r.content.decode()
+
+
+    def pickle_hidden(self, args):
+        final_args={}
+        for k, v in args.items():
+            if not is_jsonable(v):
+                x = self.store_pickle(get_dill_string(v))
+                final_args[k] = "lookup:" + x
+            else:
+                final_args[k] = v
+
+        return final_args
+
+
     @staticmethod
     def get_daisis(base_url=sandy_base_url):
         """
